@@ -151,7 +151,24 @@ int mpd_status_update(MpdObj *mi)
 		mi->CurrentState.songpos = mi->status->song;
 
 	}
+    if(mi->CurrentState.nextsongid != mi->status->nextsongid || mi->CurrentState.nextsongpos != mi->status->nextsong)
+    {
+		what_changed |= MPD_CST_NEXTSONG;
+		/* save new songid */
+		mi->CurrentState.nextsongpos = mi->status->nextsong;
+        mi->CurrentState.nextsongid = mi->status->nextsongid;
+    }
 	
+	if(mi->CurrentState.single != mi->status->single)
+	{
+		what_changed |= MPD_CST_SINGLE_MODE;
+		mi->CurrentState.single = mi->status->single;
+	}
+	if(mi->CurrentState.consume != mi->status->consume)
+	{
+		what_changed |= MPD_CST_CONSUME_MODE;
+		mi->CurrentState.consume = mi->status->consume;
+	}
 	if(mi->CurrentState.repeat != mi->status->repeat)
 	{
 		what_changed |= MPD_CST_REPEAT;
@@ -249,7 +266,14 @@ int mpd_status_update(MpdObj *mi)
             mpd_OutputEntity *output = NULL;
             mpd_sendOutputsCommand(mi->connection);
             while (( output = mpd_getNextOutput(mi->connection)) != NULL)
-            {	
+            {   
+                if(mi->num_outputs < output->id)
+                {
+                    mi->num_outputs++;
+                    mi->output_states = realloc(mi->output_states,mi->num_outputs*sizeof(int));
+                    mi->output_states[mi->num_outputs] = output->enabled;
+                    what_changed |= MPD_CST_OUTPUT;
+                }
                 if(mi->output_states[output->id] != output->enabled)
                 {
                     mi->output_states[output->id] = output->enabled;
