@@ -70,6 +70,8 @@ static void mpd_init_MpdServerState(MpdServerState *state)
 	state->state 		= -1;
 	state->songid 		= -1;     	
 	state->songpos 		= -1;
+    state->nextsongpos  = -1;
+    state->nextsongid   = -1;
 	state->dbUpdateTime 	= 0;
 	state->updatingDb 	= 0;
 	state->repeat 		= -1;
@@ -82,6 +84,8 @@ static void mpd_init_MpdServerState(MpdServerState *state)
 	state->samplerate 	= 0;
 	state->channels 	= 0;
 	state->bits 		= 0;
+    state->consume      = 0;
+    state->single       = 0;
     state->playlistLength = 0;
     state->error[0]            = '\0';
 
@@ -90,13 +94,8 @@ static void mpd_init_MpdServerState(MpdServerState *state)
 
 static MpdObj * mpd_create()
 {
-	MpdObj * mi = malloc(sizeof(*mi));
-	if( mi == NULL )
-	{
-		/* should never happen on linux */
-		return NULL;
-	}
-
+	MpdObj * mi = g_slice_new0(MpdObj);
+	g_return_val_if_fail(mi != NULL, NULL); /* should never happen on linux */
 
 	/* set default values */
 	/* we start not connected */
@@ -105,57 +104,20 @@ static MpdObj * mpd_create()
 	mi->port = 6600;
 	/* localhost */
 	mi->hostname = strdup("localhost");
-	/* no password */
-	mi->password = NULL;
 	/* 1 second timeout */
 	mi->connection_timeout = 1.0;
-	/* we have no connection pointer */
-	mi->connection = NULL;
-	/* no status */
-	mi->status = NULL;
-	/* no stats */
-	mi->stats = NULL;
-	mi->error = 0;
-	mi->error_mpd_code = 0;
-	mi->error_msg = NULL;
-	mi->CurrentSong = NULL;
 	/* info */
 	mpd_init_MpdServerState(&(mi->CurrentState));
 	mpd_init_MpdServerState(&(mi->OldState));
 
-	/**
-	 * Set signals to NULL 
-	 */
-	/* connection changed signal */
-	mi->the_connection_changed_callback = NULL;
-	mi->the_connection_changed_signal_userdata = NULL;
-
-	/* status changed */
-	mi->the_status_changed_callback = NULL;
-	mi->the_status_changed_signal_userdata = NULL;
-
-	/* error callback */
-	mi->the_error_callback = NULL;
-	mi->the_error_signal_userdata  = NULL;
 	/* connection is locked because where not connected */
 	mi->connection_lock = TRUE;
 
-	/* set queue to NULL */
-	mi->queue = NULL;
 	/* search stuff */
 	mi->search_type = MPD_SEARCH_TYPE_NONE;
 	/* no need to initialize, but set it to anything anyway*/
 	mi->search_field = MPD_TAG_ITEM_ARTIST;
 
-    /* outputs */
-    mi->num_outputs = 0;
-    mi->output_states = NULL;
-    /* set 0 */ 
-    memset((mi->supported_tags), 0,sizeof(mi->supported_tags));
-
-    mi->has_idle = 0;
-	/* commands */
-	mi->commands = NULL;
 	return mi;
 }
 
@@ -197,9 +159,14 @@ void mpd_free(MpdObj *mi)
 	{
 		mpd_freeSong(mi->CurrentSong);
 	}
+    if(mi->url_handlers)
+    {
+        g_strfreev(mi->url_handlers);
+        mi->url_handlers = NULL;
+    }
 	mpd_free_queue_ob(mi);
 	mpd_server_free_commands(mi);		
-	free(mi);
+	g_slice_free(MpdObj, mi);
 }
 
 int mpd_check_error(MpdObj *mi)
@@ -399,10 +366,29 @@ int mpd_send_password(MpdObj *mi)
 		 * connect
 		 */
 		if((mi->the_status_changed_callback != NULL))
-		{
-			mi->the_status_changed_callback( mi,
-					MPD_CST_PERMISSION, mi->the_status_changed_signal_userdata );
-		}
+        {
+            /** update the supported tags */
+            {
+                int i;
+                char **retv = mpd_server_get_tag_types(mi);
+                if(retv){
+                    for(i=0;i<MPD_TAG_ITEM_ANY;i++)
+                    {
+                        int j=0;
+                        for(j=0;retv[j] && strcasecmp(retv[j],mpdTagItemKeys[i]); j++);
+                        if(retv[j]) mi->supported_tags[i] = TRUE;
+                        else mi->supported_tags[i] = FALSE;
+                    }
+                    g_strfreev(retv);
+                }
+                /* also always true */
+                mi->supported_tags[MPD_TAG_ITEM_FILENAME] = TRUE;
+                mi->supported_tags[MPD_TAG_ITEM_ANY] = TRUE;
+            }
+            /* If permission updates, we should also call an output update, The data might be available now. */
+            mi->the_status_changed_callback( mi,
+                    MPD_CST_PERMISSION|MPD_CST_OUTPUT, mi->the_status_changed_signal_userdata );
+        }
 	}
 	return MPD_OK;
 }
@@ -556,11 +542,18 @@ int mpd_disconnect(MpdObj *mi)
 		mpd_freeSong(mi->CurrentSong);
 		mi->CurrentSong = NULL;
 	}
+    if(mi->url_handlers)
+    {
+        g_strfreev(mi->url_handlers);
+        mi->url_handlers = NULL;
+    }
 	mi->CurrentState.playlistid = -1;
 	mi->CurrentState.storedplaylistid = -1;
 	mi->CurrentState.state = -1;
 	mi->CurrentState.songid = -1;
 	mi->CurrentState.songpos = -1;
+    mi->CurrentState.nextsongid = -1;
+    mi->CurrentState.nextsongpos = -1;
 	mi->CurrentState.dbUpdateTime = 0;
 	mi->CurrentState.updatingDb = 0;
 	mi->CurrentState.repeat = -1;
@@ -681,16 +674,17 @@ int mpd_connect_real(MpdObj *mi,mpd_Connection *connection)
     {
         return retv;
     }
-/*
-
-    retv = mpd_server_update_outputs(mi);
-    if(retv != MPD_OK)
-        return retv;
-*/
-
-    retv = mpd_server_update_outputs(mi);
-    /** update the supported tags */
+    /* Trying to send password, this is needed to get right outputs and tag_types */
+    if(mi->password && strlen(mi->password) > 0)
     {
+        mpd_send_password(mi);
+    }
+    else
+    {
+        /* Update tag types, this is done in send_password.
+         * mpd_send_password() does this.
+         * So only do it when no password is send.
+         */
         int i;
         char **retv = mpd_server_get_tag_types(mi);
         if(retv){
@@ -707,6 +701,15 @@ int mpd_connect_real(MpdObj *mi,mpd_Connection *connection)
         mi->supported_tags[MPD_TAG_ITEM_FILENAME] = TRUE;
         mi->supported_tags[MPD_TAG_ITEM_ANY] = TRUE;
     }
+/*
+
+    retv = mpd_server_update_outputs(mi);
+    if(retv != MPD_OK)
+        return retv;
+*/
+
+    retv = mpd_server_update_outputs(mi);
+    /** update the supported tags */
     debug_printf(DEBUG_INFO,  "Propagating connection changed");
 
     if(mi->the_connection_changed_callback != NULL)
@@ -771,23 +774,7 @@ void	mpd_signal_connect_connection_changed(MpdObj *mi, ConnectionChangedCallback
 /* MpdData Part */
 MpdData *mpd_new_data_struct(void)
 {
-	MpdData_real* data;
-    data = g_slice_new(MpdData_real);
-	data->type = 0;
-
-	data->tag_type = 0;
-	data->tag = NULL;
-	data->song = NULL;
-	data->directory = NULL;
-	data->playlist = NULL;
-	data->output_dev = NULL;
-	data->next = NULL;
-	data->prev = NULL;
-    data->first = NULL;
-
-    data->userdata = NULL;
-    data->freefunc = NULL;
-	return (MpdData*)data;
+	return (MpdData*) g_slice_new0(MpdData_real);
 }
 
 MpdData *mpd_new_data_struct_append(MpdData  * data)
@@ -967,7 +954,7 @@ void mpd_data_free(MpdData *data)
                 data_real->freefunc(data_real->userdata);
         }
         data_real = data_real->next;
-        g_slice_free1(sizeof(*temp), temp);
+        g_slice_free(MpdData_real, temp);
     }
 }
 
@@ -995,7 +982,7 @@ static void mpd_free_queue_ob(MpdObj *mi)
             free(mi->queue->path);
         }
 
-        free(mi->queue);
+        g_slice_free(MpdQueue, mi->queue);
         mi->queue = temp;
     }
     mi->queue = NULL;
@@ -1004,13 +991,7 @@ static void mpd_free_queue_ob(MpdObj *mi)
 
 MpdQueue *mpd_new_queue_struct()
 {
-    MpdQueue* queue = malloc(sizeof(MpdQueue));
-
-    queue->type = 0;
-    queue->path = NULL;
-    queue->id = 0;
-
-    return queue;	
+    return g_slice_new0(MpdQueue);
 }
 
 
@@ -1069,7 +1050,11 @@ MpdData * mpd_server_get_output_devices(MpdObj *mi)
     mpd_finishCommand(mi->connection);
 
     /* unlock */
-    mpd_unlock_conn(mi);
+    if(mpd_unlock_conn(mi) != MPD_OK)
+    {
+        if(data)mpd_data_free(data);
+        return NULL;
+    }
     if(data == NULL) 
     {
         return NULL;
@@ -1147,30 +1132,36 @@ char ** mpd_server_get_url_handlers(MpdObj *mi)
 {
     char *temp = NULL;
     int i=0;
-    char **retv = NULL;
     if(!mpd_check_connected(mi))
     {
         debug_printf(DEBUG_WARNING,"not connected\n");
         return FALSE;
     }
+    if(mi->url_handlers){
+        return g_strdupv(mi->url_handlers);
+    }
     if(mpd_lock_conn(mi))
     {
         debug_printf(DEBUG_ERROR,"lock failed\n");
         return NULL;
     }                                           
+    /**
+     * Fetch url handlers and store them
+     */
     mpd_sendUrlHandlersCommand(mi->connection);
     while((temp = mpd_getNextHandler(mi->connection)) != NULL)
     {
-        retv = realloc(retv,(i+2)*sizeof(*retv));
-        retv[i]   = temp;
-        retv[i+1] = NULL;
+        mi->url_handlers = realloc(mi->url_handlers,(i+2)*sizeof(*mi->url_handlers));
+        mi->url_handlers[i]   = temp;
+        mi->url_handlers[i+1] = NULL;
         i++;
     } 
     mpd_finishCommand(mi->connection);
 
 
     mpd_unlock_conn(mi);
-    return retv;
+    /* Return copy */
+    return g_strdupv(mi->url_handlers);
 }
 char ** mpd_server_get_tag_types(MpdObj *mi)
 {
@@ -1236,7 +1227,7 @@ int mpd_server_update_outputs(MpdObj *mi)
     while (( output = mpd_getNextOutput(mi->connection)) != NULL)
     {	
         mi->num_outputs++;
-        mi->output_states = g_realloc(mi->output_states,mi->num_outputs*sizeof(int));
+        mi->output_states = realloc(mi->output_states,mi->num_outputs*sizeof(int));
         mi->output_states[mi->num_outputs-1] = FALSE;/*output->enabled;*/
         mpd_freeOutputElement(output);
     }
