@@ -30,19 +30,22 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "libmpdclient.h"
 
+#include "libmpdclient.h"
 #include <errno.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <stdio.h>
 #include <sys/param.h>
+
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <limits.h>
 
+#include <glib.h>
+
 #ifdef WIN32
 #  include <ws2tcpip.h>
 #  include <winsock.h>
@@ -109,9 +112,9 @@ static int do_connect_fail(mpd_Connection *connection,
 	ioctlsocket(connection->sock, FIONBIO, (u_long FAR*) &iMode);
 	return 0;
 }
-#else /* !WIN32 (sane operating systems) */
+#elif defined(__solaris__) || defined(__sun__)
 static int do_connect_fail(mpd_Connection *connection,
-                           const struct sockaddr *serv_addr, int addrlen)
+		const struct sockaddr *serv_addr, int addrlen)
 {
 	int flags;
 	if (connect(connection->sock, serv_addr, addrlen) < 0)
@@ -120,6 +123,50 @@ static int do_connect_fail(mpd_Connection *connection,
 	fcntl(connection->sock, F_SETFL, flags | O_NONBLOCK);
 	return 0;
 }
+#else /* !WIN32 (sane operating systems) */
+static int do_connect_fail(mpd_Connection *connection,
+                           const struct sockaddr *serv_addr, int addrlen)
+{
+	int flags, res,valopt;
+	struct timeval tv;
+	fd_set myset;
+	flags = fcntl(connection->sock, F_GETFL, 0);
+	fcntl(connection->sock, F_SETFL, flags | O_NONBLOCK);
+	/*if (connect(connection->sock, serv_addr, addrlen) < 0)
+		return 1;*/
+    /* Do a non blocking connect */
+    res = connect(connection->sock, serv_addr, addrlen);
+    if (res < 0) { 
+        if (errno == EINPROGRESS) { 
+            tv.tv_sec = connection->timeout.tv_sec;
+            tv.tv_usec = connection->timeout.tv_usec;
+            FD_ZERO(&myset); 
+            FD_SET(connection->sock, &myset); 
+            if (select(connection->sock+1, NULL, &myset, NULL, &tv) > 0) { 
+                socklen_t lon; 
+                lon = sizeof(int); 
+                /* Check for errors */
+                getsockopt(connection->sock, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon); 
+                if (valopt) { 
+                    fprintf(stderr, "Error in connection() %d - %s\n", valopt, strerror(valopt)); 
+                    return 1;
+                } 
+            } 
+            else { 
+                /* Connecting timed out. */
+                fprintf(stderr, "Timeout or error() %d - %s\n", valopt, strerror(valopt)); 
+                return 1;
+            } 
+        } 
+        else { 
+            /* Failed to connect */
+            fprintf(stderr, "Error connecting %d - %s\n", errno, strerror(errno)); 
+            return 1;
+        } 
+    } 
+
+	return 0;
+}
 #endif /* !WIN32 */
 
 #ifdef MPD_HAVE_GAI
@@ -298,7 +345,7 @@ static char * mpd_sanitizeArg(const char * arg) {
 
 static mpd_ReturnElement * mpd_newReturnElement(const char * name, const char * value)
 {
-	mpd_ReturnElement * ret = malloc(sizeof(mpd_ReturnElement));
+	mpd_ReturnElement* ret = g_slice_new(mpd_ReturnElement);
 
 	ret->name = strdup(name);
 	ret->value = strdup(value);
@@ -309,7 +356,7 @@ static mpd_ReturnElement * mpd_newReturnElement(const char * name, const char *
 static void mpd_freeReturnElement(mpd_ReturnElement * re) {
 	free(re->name);
 	free(re->value);
-	free(re);
+	g_slice_free(mpd_ReturnElement, re);
 }
 
 void mpd_setConnectionTimeout(mpd_Connection * connection, float timeout) {
@@ -404,21 +451,12 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	int err;
 	char * rt;
 	char * output =  NULL;
-	mpd_Connection * connection = malloc(sizeof(mpd_Connection));
+	mpd_Connection * connection = g_slice_new0(mpd_Connection);
 	struct timeval tv;
 	fd_set fds;
 	strcpy(connection->buffer,"");
 	connection->sock = -1;
-	connection->buflen = 0;
-	connection->bufstart = 0;
 	strcpy(connection->errorStr,"");
-	connection->error = 0;
-	connection->doneProcessing = 0;
-	connection->commandList = 0;
-	connection->listOks = 0;
-	connection->doneListOk = 0;
-	connection->returnElement = NULL;
-	connection->request = NULL;
 
 	if (winsock_dll_error(connection))
 		return connection;
@@ -493,7 +531,7 @@ void mpd_closeConnection(mpd_Connection * connection) {
 	closesocket(connection->sock);
 	if(connection->returnElement) free(connection->returnElement);
 	if(connection->request) free(connection->request);
-	free(connection);
+	g_slice_free(mpd_Connection, connection);
 	WSACleanup();
 }
 
@@ -721,8 +759,6 @@ void mpd_sendStatusCommand(mpd_Connection * connection) {
 }
 
 mpd_Status * mpd_getStatus(mpd_Connection * connection) {
-	mpd_Status * status;
-
 	/*mpd_executeCommand(connection,"status\n");
 
 	if(connection->error) return NULL;*/
@@ -735,28 +771,18 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 
 	if(!connection->returnElement) mpd_getNextReturnElement(connection);
 
-	status = malloc(sizeof(mpd_Status));
+	mpd_Status* status = g_slice_new0(mpd_Status);
 	status->volume = -1;
-	status->repeat = 0;
-	status->random = 0;
 	status->playlist = -1;
-    status->storedplaylist = -1;
+	status->storedplaylist = -1;
 	status->playlistLength = -1;
 	status->state = -1;
-	status->song = 0;
-	status->songid = 0;
-	status->elapsedTime = 0;
-	status->totalTime = 0;
-	status->bitRate = 0;
-	status->sampleRate = 0;
-	status->bits = 0;
-	status->channels = 0;
+	status->nextsong = -1;
+	status->nextsongid = -1;
 	status->crossfade = -1;
-	status->error = NULL;
-	status->updatingDb = 0;
 
 	if(connection->error) {
-		free(status);
+		g_slice_free(mpd_Status, status);
 		return NULL;
 	}
 	while(connection->returnElement) {
@@ -767,6 +793,12 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 		else if(strcmp(re->name,"repeat")==0) {
 			status->repeat = atoi(re->value);
 		}
+		else if(strcmp(re->name,"single")==0) {
+			status->single = atoi(re->value);
+		}
+		else if(strcmp(re->name,"consume")==0) {
+			status->consume = atoi(re->value);
+		}
 		else if(strcmp(re->name,"random")==0) {
 			status->random = atoi(re->value);
 		}
@@ -799,6 +831,12 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 		else if(strcmp(re->name,"songid")==0) {
 			status->songid = atoi(re->value);
 		}
+		else if(strcmp(re->name,"nextsong")==0) {
+			status->nextsong = atoi(re->value);
+		}
+		else if(strcmp(re->name,"nextsongid")==0) {
+			status->nextsongid = atoi(re->value);
+		}
 		else if(strcmp(re->name,"time")==0) {
 			char * tok = strchr(re->value,':');
 			/* the second strchr below is a safety check */
@@ -830,19 +868,19 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 
 		mpd_getNextReturnElement(connection);
 		if(connection->error) {
-			free(status);
+			g_slice_free(mpd_Status, status);
 			return NULL;
 		}
 	}
 
 	if(connection->error) {
-		free(status);
+		g_slice_free(mpd_Status, status);
 		return NULL;
 	}
 	else if(status->state<0) {
 		strcpy(connection->errorStr,"state not found");
 		connection->error = 1;
-		free(status);
+		g_slice_free(mpd_Status, status);
 		return NULL;
 	}
 
@@ -851,7 +889,7 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 
 void mpd_freeStatus(mpd_Status * status) {
 	if(status->error) free(status->error);
-	free(status);
+	g_slice_free(mpd_Status, status);
 }
 
 void mpd_sendStatsCommand(mpd_Connection * connection) {
@@ -859,8 +897,6 @@ void mpd_sendStatsCommand(mpd_Connection * connection) {
 }
 
 mpd_Stats * mpd_getStats(mpd_Connection * connection) {
-	mpd_Stats * stats;
-
 	/*mpd_executeCommand(connection,"stats\n");
 
 	if(connection->error) return NULL;*/
@@ -873,17 +909,10 @@ mpd_Stats * mpd_getStats(mpd_Connection * connection) {
 
 	if(!connection->returnElement) mpd_getNextReturnElement(connection);
 
-	stats = malloc(sizeof(mpd_Stats));
-	stats->numberOfArtists = 0;
-	stats->numberOfAlbums = 0;
-	stats->numberOfSongs = 0;
-	stats->uptime = 0;
-	stats->dbUpdateTime = 0;
-	stats->playTime = 0;
-	stats->dbPlayTime = 0;
+	mpd_Stats* stats = g_slice_new0(mpd_Stats);
 
 	if(connection->error) {
-		free(stats);
+		mpd_freeStats(stats);
 		return NULL;
 	}
 	while(connection->returnElement) {
@@ -912,13 +941,13 @@ mpd_Stats * mpd_getStats(mpd_Connection * connection) {
 
 		mpd_getNextReturnElement(connection);
 		if(connection->error) {
-			free(stats);
+			mpd_freeStats(stats);
 			return NULL;
 		}
 	}
 
 	if(connection->error) {
-		free(stats);
+		mpd_freeStats(stats);
 		return NULL;
 	}
 
@@ -926,14 +955,11 @@ mpd_Stats * mpd_getStats(mpd_Connection * connection) {
 }
 
 void mpd_freeStats(mpd_Stats * stats) {
-	free(stats);
+	g_slice_free(mpd_Stats, stats);
 }
 
 mpd_SearchStats * mpd_getSearchStats(mpd_Connection * connection)
 {
-	mpd_SearchStats * stats;
-	mpd_ReturnElement * re;
-
 	if (connection->doneProcessing ||
 	    (connection->listOks && connection->doneListOk)) {
 		return NULL;
@@ -944,9 +970,8 @@ mpd_SearchStats * mpd_getSearchStats(mpd_Connection * connection)
 	if (connection->error)
 		return NULL;
 
-	stats = malloc(sizeof(mpd_SearchStats));
-	stats->numberOfSongs = 0;
-	stats->playTime = 0;
+	mpd_ReturnElement* re;
+	mpd_SearchStats* stats = g_slice_new0(mpd_SearchStats);
 
 	while (connection->returnElement) {
 		re = connection->returnElement;
@@ -959,13 +984,13 @@ mpd_SearchStats * mpd_getSearchStats(mpd_Connection * connection)
 
 		mpd_getNextReturnElement(connection);
 		if (connection->error) {
-			free(stats);
+			mpd_freeSearchStats(stats);
 			return NULL;
 		}
 	}
 
 	if (connection->error) {
-		free(stats);
+		mpd_freeSearchStats(stats);
 		return NULL;
 	}
 
@@ -974,28 +999,7 @@ mpd_SearchStats * mpd_getSearchStats(mpd_Connection * connection)
 
 void mpd_freeSearchStats(mpd_SearchStats * stats)
 {
-	free(stats);
-}
-
-static void mpd_initSong(mpd_Song * song) {
-	song->file = NULL;
-	song->artist = NULL;
-	song->album = NULL;
-	song->track = NULL;
-	song->title = NULL;
-	song->name = NULL;
-	song->date = NULL;
-	/* added by Qball */
-	song->genre = NULL;
-	song->composer = NULL;
-	song->performer = NULL;
-	song->disc = NULL;
-	song->comment = NULL;
-    song->albumartist = NULL;
-
-	song->time = MPD_SONG_NO_TIME;
-	song->pos = MPD_SONG_NO_NUM;
-	song->id = MPD_SONG_NO_ID;
+	g_slice_free(mpd_SearchStats, stats);
 }
 
 static void mpd_finishSong(mpd_Song * song) {
@@ -1015,16 +1019,16 @@ static void mpd_finishSong(mpd_Song * song) {
 }
 
 mpd_Song * mpd_newSong(void) {
-	mpd_Song * ret = malloc(sizeof(mpd_Song));
-
-	mpd_initSong(ret);
-
+	mpd_Song * ret = g_slice_new0(mpd_Song);
+	ret->time = MPD_SONG_NO_TIME;
+	ret->pos = MPD_SONG_NO_NUM;
+	ret->id = MPD_SONG_NO_ID;
 	return ret;
 }
 
 void mpd_freeSong(mpd_Song * song) {
 	mpd_finishSong(song);
-	free(song);
+	g_slice_free(mpd_Song, song);
 }
 
 mpd_Song * mpd_songDup(const mpd_Song * song) {
@@ -1050,26 +1054,18 @@ mpd_Song * mpd_songDup(const mpd_Song * song) {
 	return ret;
 }
 
-static void mpd_initDirectory(mpd_Directory * directory) {
-	directory->path = NULL;
-}
 
 static void mpd_finishDirectory(mpd_Directory * directory) {
 	if(directory->path) free(directory->path);
 }
 
 mpd_Directory * mpd_newDirectory(void) {
-	mpd_Directory * directory = malloc(sizeof(mpd_Directory));;
-
-	mpd_initDirectory(directory);
-
-	return directory;
+	return g_slice_new0(mpd_Directory);
 }
 
 void mpd_freeDirectory(mpd_Directory * directory) {
 	mpd_finishDirectory(directory);
-
-	free(directory);
+	g_slice_free(mpd_Directory, directory);
 }
 
 mpd_Directory * mpd_directoryDup(mpd_Directory * directory) {
@@ -1080,27 +1076,18 @@ mpd_Directory * mpd_directoryDup(mpd_Directory * directory) {
 	return ret;
 }
 
-static void mpd_initPlaylistFile(mpd_PlaylistFile * playlist) {
-	playlist->path = NULL;
-    playlist->mtime = NULL;
-}
-
 static void mpd_finishPlaylistFile(mpd_PlaylistFile * playlist) {
 	if(playlist->path) free(playlist->path);
     if(playlist->mtime) free(playlist->mtime);
 }
 
 mpd_PlaylistFile * mpd_newPlaylistFile(void) {
-	mpd_PlaylistFile * playlist = malloc(sizeof(mpd_PlaylistFile));
-
-	mpd_initPlaylistFile(playlist);
-
-	return playlist;
+	return g_slice_new0(mpd_PlaylistFile);
 }
 
 void mpd_freePlaylistFile(mpd_PlaylistFile * playlist) {
 	mpd_finishPlaylistFile(playlist);
-	free(playlist);
+	g_slice_free(mpd_PlaylistFile, playlist);
 }
 
 mpd_PlaylistFile * mpd_playlistFileDup(mpd_PlaylistFile * playlist) {
@@ -1112,10 +1099,6 @@ mpd_PlaylistFile * mpd_playlistFileDup(mpd_PlaylistFile * playlist) {
 	return ret;
 }
 
-static void mpd_initInfoEntity(mpd_InfoEntity * entity) {
-	entity->info.directory = NULL;
-}
-
 static void mpd_finishInfoEntity(mpd_InfoEntity * entity) {
 	if(entity->info.directory) {
 		if(entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
@@ -1131,16 +1114,12 @@ static void mpd_finishInfoEntity(mpd_InfoEntity * entity) {
 }
 
 mpd_InfoEntity * mpd_newInfoEntity(void) {
-	mpd_InfoEntity * entity = malloc(sizeof(mpd_InfoEntity));
-
-	mpd_initInfoEntity(entity);
-
-	return entity;
+	return g_slice_new0(mpd_InfoEntity);
 }
 
 void mpd_freeInfoEntity(mpd_InfoEntity * entity) {
 	mpd_finishInfoEntity(entity);
-	free(entity);
+	g_slice_free(mpd_InfoEntity, entity);
 }
 
 static void mpd_sendInfoCommand(mpd_Connection * connection,const char * command) {
@@ -1628,18 +1607,18 @@ void mpd_sendSwapIdCommand(mpd_Connection * connection, int id1, int id2) {
 	free(string);
 }
 
-void mpd_sendSeekCommand(mpd_Connection * connection, int song, int time) {
+void mpd_sendSeekCommand(mpd_Connection * connection, int song, int seek_time) {
 	int len = strlen("seek")+2+INTLEN+3+INTLEN+3;
 	char *string = malloc(len);
-	snprintf(string, len, "seek \"%i\" \"%i\"\n", song, time);
+	snprintf(string, len, "seek \"%i\" \"%i\"\n", song, seek_time);
 	mpd_sendInfoCommand(connection,string);
 	free(string);
 }
 
-void mpd_sendSeekIdCommand(mpd_Connection * connection, int id, int time) {
+void mpd_sendSeekIdCommand(mpd_Connection * connection, int id, int seek_time) {
 	int len = strlen("seekid")+2+INTLEN+3+INTLEN+3;
 	char *string = malloc(len);
-	snprintf(string, len, "seekid \"%i\" \"%i\"\n", id, time);
+	snprintf(string, len, "seekid \"%i\" \"%i\"\n", id, seek_time);
 	mpd_sendInfoCommand(connection,string);
 	free(string);
 }
@@ -1671,6 +1650,22 @@ void mpd_sendPrevCommand(mpd_Connection * connection) {
 	mpd_executeCommand(connection,"previous\n");
 }
 
+void mpd_sendSingleCommand(mpd_Connection * connection, int singleMode) {
+	int len = strlen("repeat")+2+INTLEN+3;
+	char *string = malloc(len);
+	snprintf(string, len, "single \"%i\"\n", singleMode);
+	mpd_executeCommand(connection,string);
+	free(string);
+}
+
+void mpd_sendConsumeCommand(mpd_Connection * connection, int consumeMode) {
+	int len = strlen("repeat")+2+INTLEN+3;
+	char *string = malloc(len);
+	snprintf(string, len, "consume \"%i\"\n", consumeMode);
+	mpd_executeCommand(connection,string);
+	free(string);
+}
+
 void mpd_sendRepeatCommand(mpd_Connection * connection, int repeatMode) {
 	int len = strlen("repeat")+2+INTLEN+3;
 	char *string = malloc(len);
@@ -1695,14 +1690,6 @@ void mpd_sendSetvolCommand(mpd_Connection * connection, int volumeChange) {
 	free(string);
 }
 
-void mpd_sendVolumeCommand(mpd_Connection * connection, int volumeChange) {
-	int len = strlen("volume")+2+INTLEN+3;
-	char *string = malloc(len);
-	snprintf(string, len, "volume \"%i\"\n", volumeChange);
-	mpd_executeCommand(connection,string);
-	free(string);
-}
-
 void mpd_sendCrossfadeCommand(mpd_Connection * connection, int seconds) {
 	int len = strlen("crossfade")+2+INTLEN+3;
 	char *string = malloc(len);
@@ -1757,8 +1744,6 @@ void mpd_sendOutputsCommand(mpd_Connection * connection) {
 }
 
 mpd_OutputEntity * mpd_getNextOutput(mpd_Connection * connection) {
-	mpd_OutputEntity * output = NULL;
-
 	if(connection->doneProcessing || (connection->listOks &&
 				connection->doneListOk))
 	{
@@ -1767,10 +1752,8 @@ mpd_OutputEntity * mpd_getNextOutput(mpd_Connection * connection) {
 
 	if(connection->error) return NULL;
 
-	output = malloc(sizeof(mpd_OutputEntity));
+	mpd_OutputEntity* output = g_slice_new0(mpd_OutputEntity);
 	output->id = -10;
-	output->name = NULL;
-	output->enabled = 0;
 
 	if(!connection->returnElement) mpd_getNextReturnElement(connection);
 
@@ -1789,10 +1772,9 @@ mpd_OutputEntity * mpd_getNextOutput(mpd_Connection * connection) {
 
 		mpd_getNextReturnElement(connection);
 		if(connection->error) {
-			free(output);
+			mpd_freeOutputElement(output);
 			return NULL;
 		}
-
 	}
 
 	return output;
@@ -1815,8 +1797,9 @@ void mpd_sendDisableOutputCommand(mpd_Connection * connection, int outputId) {
 }
 
 void mpd_freeOutputElement(mpd_OutputEntity * output) {
-	free(output->name);
-	free(output);
+	if(output->name)
+		free(output->name);
+	g_slice_free(mpd_OutputEntity, output);
 }
 
 /**
